import { FieldPacket } from "mysql2/promise";
import { AdEntity, NewAdEntity, SimpleAdEntity } from "../types";
import { pool } from "../utils/db";
import { ValidationError } from "../utils/error";
import { v4 as uuid } from "uuid";



type AdRecordResults = [AdRecord[], FieldPacket[]];

export class AdRecord implements AdEntity {
    public id: string;
    public name: string;
    public description: string;
    public price: number;
    public url: string;
    public lat: number;
    public lon: number; 

    constructor(obj: AdEntity) {
        if(!obj.name || obj.name.length > 100) {
            throw new ValidationError("Nazwa ogłosznia nie może być pusta ani mieć więcej niż 100 znaków")
        }
        if(obj.description.length > 1000) {
            throw new ValidationError("Treść ogłoszenia nie może być dłuższa niż 1000 znaków")
        }
        if(obj.price < 0 || obj.price > 999999) {
            throw new ValidationError("Cena musi mieścić się w zakresie 0 - 999999")
        }
        if(!obj.url || obj.url.length > 100) {
            throw new ValidationError("Link nie może byc ani pusty ani dłuższy 100")
        }
        if (typeof obj.lat !== 'number' || typeof obj.lon !== "number") {
            throw new ValidationError("Współrzędne muszą być liczbą")
        }

        this.id = obj.id;
        this.name = obj.name;
        this.description = obj.description;
        this.price = obj.price;
        this.url = obj.url;
        this.lat= obj.lat;
        this.lon = obj.lon;
    }


    static async getOne(id: string): Promise<AdRecord | null> {
        const [results] = await pool.execute("SELECT * FROM `ads` WHERE id = :id", {
            id: id,
        }) as AdRecordResults;
       
       return results.length === 0 ? null : new AdRecord(results[0])
    }

    static async insert(obj: NewAdEntity): Promise<string> {

        if(!obj.id) {
            obj.id = uuid();
        };

        await pool.execute("INSERT INTO `ads` VALUES (:id, :name, :description, :price, :url, :lat, :lon)", {
            id: obj.id,
            name: obj.name,
            description: obj.description,
            price: obj.price,
            url: obj.url,
            lat: obj.lat,
            lon: obj.lon,
        });

        return obj.id
    }

    static async listAll(): Promise<SimpleAdEntity[]> {
        
        const [results] = await pool.execute("SELECT * FROM `ads` ORDER BY `name` DESC") as AdRecordResults;
        console.log(results);
        
        return results.map(result => {
            const {
                id, lat, lon,
            } = result;

            return {
                id, lat, lon,
            }
        });
    }

    static async findAll(name: string) : Promise<SimpleAdEntity[]> {
        const [results] = await pool.execute("SELECT * FROM `ads` WHERE `name` LIKE :search", {
            search: `%${name}%`,

        }) as AdRecordResults;

        return results.map(result => {
            const {
                id, lat, lon,
            } = result;

            return {
                id, lat, lon,
            }
        });
    }
};
